import java.util.ArrayList;
import java.util.Collections;
import java.io.*; 

/******************************************************************************
*  Spell database index for the Wizard Spell Compendium.
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2017-06-23
*  @version  1.03
******************************************************************************/

public class SpellIndex {

	//--------------------------------------------------------------------------
	//  Constants
	//--------------------------------------------------------------------------

	/** Name of file with record data. */
	static final String DATA_FILE = "WizardSpellCompendiumIndex.csv";

	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** The singleton class instance. */
	static SpellIndex instance = null;
	
	/** Array of individual records. */
	ArrayList<Spell> array;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor (read from dedicated file).
	*/
	protected SpellIndex () throws IOException {
		String[][] table = CSVReader.readFile(DATA_FILE);
		array = new ArrayList<Spell>();
		for (int i = 1; i < table.length; i++)
			array.add(new Spell(table[i]));
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------

	/**
	*  Access the singleton class instance.
	*/
	public static SpellIndex getInstance() {
		if (instance == null) {
			try {
				instance = new SpellIndex();
			}
			catch (IOException e) {
				System.err.println("Failed to read file " + DATA_FILE);
			}
		}
		return instance;
	}

	/**
	*  Get size of the spell index.
	*/
	public int size() {
		return array.size();
	}

	/**
	*  Get spell by index.
	*/
	public Spell get (int idx) {
		return array.get(idx);	
	}

	/**
	*  Get spell by name.
	*/
	public Spell get (String name) {
		for (Spell s: array)
			if (s.getName().equals(name))
				return s;		
		return null;
	}

	/**
	*  Compile list of duplicate names.
	*/
	public ArrayList<String> compileDuplicateNames () {
		ArrayList<String>	allNames = new ArrayList<String>();
		ArrayList<String> dupNames = new ArrayList<String>();
		for (Spell spell: array) {
			String name = spell.getName();
			if (!isStringInList(name, allNames))
				allNames.add(name);
			else
				dupNames.add(name);
		}
		Collections.sort(dupNames);
		return dupNames;
	}

	/**
	*  Compile list of all origin specifiers.
	*/
	public ArrayList<String> compileOriginList () {
		ArrayList<String>	list = new ArrayList<String>();
		for (Spell spell: array) {
			if (spell.hasOrigin()) {
				String origin = spell.getOrigin();
				if (!isStringInList(origin, list))
					list.add(origin);
			}
		}
		Collections.sort(list);
		return list;
	}

	/**
	*  Compile list of all school specifiers.
	*/
	public ArrayList<String> compileSchoolList () {
		ArrayList<String>	list = new ArrayList<String>();
		for (Spell spell: array) {
			String[] schools = spell.getSchools().split(", ");
			for (String s: schools)
				if (!isStringInList(s, list))
					list.add(s);
		}
		Collections.sort(list);
		return list;
	}

	/**
	*  Is this string in a given list?
	*/
	boolean isStringInList (String string, ArrayList<String> list) {
		for (String s: list)
			if (s.equals(string))
				return true;
		return false;
	}

	/**
	*  Print index to CSV file, throwing exceptions.
	*/
	private void _printToCSVFile (String filename) throws IOException {
		FileWriter fw = new FileWriter(filename);
      PrintWriter printOut = new PrintWriter(fw);
		printOut.println(Spell.getCSVHeader());
		for (Spell s: array)
			printOut.println(s.toCSVString());
		printOut.close();
	}

	/**
	*  Print index to CSV file, and catch exceptions.
	*/
	public void printToCSVFile (String filename) {
		try {
			_printToCSVFile(filename);
		}
		catch (IOException e) {
			System.err.println("Failed to write file " + filename);
		}
	}

	/**
	*  Main test method.
	*/
	public static void main (String[] args) {

		// Test reading database file
		System.out.println("Reading file: " + DATA_FILE);
		SpellIndex index = SpellIndex.getInstance();

		// Database statistics & integrity checks
		System.out.println("Total spells: " + index.size());
		ArrayList<String> dupList = index.compileDuplicateNames();
		System.out.println("Duplicated names: " + dupList.size() + " " + dupList);
		ArrayList<String> originList = index.compileOriginList();
		System.out.println("Special origins: " + originList.size() + " " + originList);
		ArrayList<String> schoolList = index.compileSchoolList();
		System.out.println("Schools of magic: " + schoolList.size() + " " + schoolList);
		
		// Write out modified database
		//index.printToCSVFile("NewData.csv");
	}
}

